@@ -1,17 +1,24 @@
-from .slash.tools import ParseMethod, handle_options, handle_thing, resolve
-from .slash.http import create_global_command, create_guild_command, delete_global_command, delete_guild_command, delete_guild_commands, edit_global_command, edit_guild_command, get_command, get_global_commands, get_guild_commands, delete_global_commands, get_id
-from .slash.types import MessageCommand, OptionType, SlashCommand, SlashOption, SlashPermission, SubSlashCommand, SubSlashCommandGroup, UserCommand
-from .tools import MISSING, get, get_index
+from .slash.errors import NoAsyncCallback
+from .errors import MissingListenedComponentParameters, WrongType
+from .slash.tools import ParseMethod, cache_data, format_name, handle_options, handle_thing
+from .slash.http import create_global_command, create_guild_command, delete_global_command, delete_guild_command, delete_guild_commands, edit_global_command, edit_guild_command, get_command, get_command_permissions, get_global_commands, get_guild_commands, delete_global_commands, get_id, update_command_permissions
+from .slash.types import AdditionalType, ContextCommand, MessageCommand, OptionType, SlashCommand, SlashOption, SubSlashCommand, SubSlashCommandGroup, UserCommand
+from .tools import MISSING, _or, get_index, setup_logger
 from .http import jsonifyMessage, BetterRoute, send_files
-from .receive import EphemeralComponent, Message, SlashedContext, WebhookMessage, SlashedCommand, SlashedSubCommand, getResponseMessage
+from .receive import Interaction, Message, SlashedContext, WebhookMessage, SlashedCommand, SlashedSubCommand, getResponseMessage
 
 import discord
-from discord.errors import Forbidden, InvalidArgument
 from discord.ext import commands as com
+from discord.errors import Forbidden, InvalidArgument
 
+import zlib
+import json
+import inspect
 import asyncio
 from typing import Dict, List, Tuple, Union
 
+logging = setup_logger(__name__)
+
 class Slash():
     """
     A class for using slash commands
@@ -30,6 +37,12 @@ class Slash():
         wait_sync: :class:`float`, optional
             How many seconds will be waited until the commands are going to be synchronized; Default ``1``
 
+        auto_defer: Tuple[:class:`bool`, :class:`bool`]
+            Settings for the auto-defer; Default ``(True, False)``
+
+            ``[0]``: Whether interactions should be deferred automatically
+
+            ``[1]``: Whether the deferration should be hidden (True) or public (False)
 
     Example
     ------------------
@@ -70,7 +83,7 @@ class Slash():
         
 
     """
-    def __init__(self, client, parse_method = ParseMethod.AUTO, delete_unused = False, wait_sync = 1) -> None:
+    def __init__(self, client, parse_method = ParseMethod.AUTO, delete_unused = False, wait_sync = 1, auto_defer = True) -> None:
         """Creates a new slash command thing
         
         Example
@@ -78,26 +91,42 @@ class Slash():
         Slash(client)
         ```
         """
+        self._buffer = bytearray()
+        self._zlib = zlib.decompressobj()
+
         self.parse_method: int = parse_method
         self.delete_unused: bool = delete_unused
         self.wait_sync: float = wait_sync
+        self.auto_defer: Tuple[bool, bool] = auto_defer if type(auto_defer) in [list, tuple] else (auto_defer, False)
 
         self._discord: com.Bot = client
         self.commands: Dict[(str, SlashCommand)] = {}
         self.subcommands: Dict[(str, Dict[(str, SubSlashCommand)])] = {}
-        self.subcommand_groups: Dict[(str, Dict[(str, SubSlashCommandGroup)])] = {}
-        self.context_commands = {"message": {}, "user": {}}
-        self._discord.add_listener(self._on_socket_response, 'on_socket_response')
+        self.context_commands: Dict[str, ContextCommand] = {"message": {}, "user": {}}
+        if discord.__version__.startswith("2"):
+            self._discord.add_listener(self._on_response, "on_socket_raw_receive")
+        elif discord.__version__.startswith("1"):
+            self._discord.add_listener(self._on_response, 'on_socket_response')
 
         self.ready = False
         async def client_ready():
-            await asyncio.sleep(self.wait_sync or 1)
+            await asyncio.sleep(_or(self.wait_sync, 1))
             self._discord.loop.create_task(self.add_commands())
             self.ready = True
         self._discord.add_listener(client_ready, "on_ready")
 
-    async def _on_socket_response(self, msg):
-        """Will be executed if the bot receives a socket response"""
+    async def _on_response(self, msg):
+        if discord.__version__.startswith("2"):
+            if type(msg) is bytes:
+                self._buffer.extend(msg)
+
+                if len(msg) < 4 or msg[-4:] != b'\x00\x00\xff\xff':
+                    return
+                msg = self._zlib.decompress(self._buffer)
+                msg = msg.decode('utf-8')
+                self._buffer = bytearray()
+                msg = json.loads(msg)
+        
         if msg["t"] != "INTERACTION_CREATE":
             return
         data = msg["d"]
@@ -105,157 +134,156 @@ class Slash():
         if int(data["type"]) not in [1, 2]:
             return
 
-        guild = await self._discord.fetch_guild(data["guild_id"])
+        guild = cache_data(data["guild_id"], AdditionalType.GUILD, data, self._discord._connection)
         user = discord.Member(data=data["member"], guild=guild, state=self._discord._connection)
-        channel = await self._discord.fetch_channel(data["channel_id"])
+        channel = await handle_thing(data["channel_id"], OptionType.CHANNEL, data, self.parse_method, self._discord)
 
-        if data["data"]["type"] == 1:
+        interaction = Interaction(self._discord._connection, data, user)
+        if self.auto_defer[0] is True:
+            await interaction.defer(self.auto_defer[1])
+        self._discord.dispatch("interaction_received", interaction)
+
+        #region basic commands
+        if data["data"]["type"] == 1 and not (data["data"].get("options") and data["data"]["options"][0]["type"] in [OptionType.SUB_COMMAND, OptionType.SUB_COMMAND_GROUP]):
             x = self.commands.get(data["data"]["name"])
-            if x:
+            if x is not None:
                 options = {}
                 if data["data"].get("options") is not None:
                     options = await handle_options(data, data["data"]["options"], self.parse_method, self._discord)
-                await x.callback(SlashedCommand(self._discord, command=x, data=data, user=user, channel=channel, guild_ids=x.guild_ids), **options)
+                context = SlashedCommand(self._discord, command=x, data=data, user=user, channel=channel, guild_ids=x.guild_ids)
+                # Handle autodefer
+                context._handle_auto_defer(self.auto_defer)
+
+                await x.callback(context, **options)
                 return
         elif data["data"]["type"] == 2:
             x = self.context_commands["user"].get(data["data"]["name"])
-            if x:
+            if x is not None:
                 member = await handle_thing(data["data"]["target_id"], OptionType.MEMBER, data, self.parse_method, self._discord)
                 await x.callback(SlashedContext(self._discord, command=x, data=data, user=user, channel=channel, guild_ids=x.guild_ids), member)
+                return
         elif data["data"]["type"] == 3:
             x = self.context_commands["message"].get(data["data"]["name"])
-            if x:
+            if x is not None:
                 message = await handle_thing(data["data"]["target_id"], 44, data, self.parse_method, self._discord)
-                await x.callback(SlashedContext(self._discord, command=x, data=data, user=user, channel=channel, guild_ids=x.guild_ids), message)
-    
-        fixed_options = []
-        x_base = self.subcommands.get(data["data"]["name"]) or self.subcommand_groups.get(data["data"]["name"])
-        if x_base:
-            x = x_base.get(data["data"]["options"][0]["name"])
+                context = SlashedContext(self._discord, command=x, data=data, user=user, channel=channel, guild_ids=x.guild_ids)
+                # Handle autodefer
+                context._handle_auto_defer(self.auto_defer)
 
+                await x.callback(context, message)
+                return
+        #endregion
+
+        fixed_options = []
+        x_base = self.subcommands.get(data["data"]["name"])
+        if x_base:
             op = data["data"]["options"][0]
             while op["type"] != 1:
                 op = op["options"][0]
             fixed_options = op.get("options", [])
             
+            x = x_base.get(data["data"]["options"][0]["name"])
+            if type(x) is dict:
+                x = x.get(data["data"]["options"][0]["options"][0]["name"])
+
             if x is None:
                 x = x_base.get(op["name"])
-                
-            options = await handle_options(data, fixed_options, self.self.parse_method, self._discord)
+            options = await handle_options(data, fixed_options, self.parse_method, self._discord)
+
             if x:
-                await x.callback(SlashedSubCommand(self._discord, x, data, user, channel, x.guild_ids), **options)
+                context = SlashedSubCommand(self._discord, x, data, user, channel, x.guild_ids)
+                if self.auto_defer[0] is True:
+                    await context.defer(self.auto_defer[1])
+                await x.callback(context, **options)
                 return
 
     async def add_commands(self):
-        commands = {
+        added_commands = {
             "globals": {},
             "guilds": {}
         }
         own_guild_ids = [str(x.id) for x in self._discord.guilds]
         
-        #region contgext commands
-        for command in list(self.context_commands["message"].values()) + list(self.context_commands["user"].values()):
-            if command.guild_ids is not MISSING:        
-                for x in (command.guild_ids or command.guild_permissions.keys()):
-                    if str(x) not in own_guild_ids:
-                        raise InvalidArgument("Client is not in a server with the id '" + str(x) + "'")
-                
-                    if commands["guilds"].get(x) is None:
-                        commands["guilds"][x] = {}
-
-                    if command.guild_permissions is not MISSING:
-                        command.permissions = command.guild_permissions.get(x)
-                    await self.add_guild_command(command, x)
-                    commands["guilds"][x][command.name] = command
-            else:
-                await self.add_global_command(command)
-                commands["globals"][command.name] = command
+        #region gather commands
+        commands = self.commands
+        for x in self.subcommands:
+            for y in self.subcommands[x]:
+                sub = self.subcommands[x][y]
+                if type(sub) is dict:
+                    for z in self.subcommands[x][y]:
+                        group = self.subcommands[x][y][z]
+                        if commands.get(group.base_names[0]) is not None:
+                            index = get_index(commands[group.base_names[0]].options, group.base_names[1], lambda x: getattr(x, "name"))
+                            if index > -1:
+                                _ops = commands[group.base_names[0]].options[index].options
+                                _ops.append(group.to_option())
+                                commands[group.base_names[0]].options[index].options = _ops
+                            else:
+                                _ops = commands[group.base_names[0]].options
+                                _ops.append(SlashOption(OptionType.SUB_COMMAND_GROUP, group.base_names[1], options=[group.to_option()]))
+                                commands[group.base_names[0]].options = _ops
+                        else:
+                            commands[group.base_name] = SlashCommand(None, group.base_names[0], MISSING, [
+                                    SlashOption(OptionType.SUB_COMMAND_GROUP, group.base_names[1], options=[group.to_option()])
+                                ],
+                                guild_ids=group.guild_ids, default_permission=group.default_permission, guild_permissions=group.guild_permission)
+                else:
+                    if commands.get(sub.base_name) is not None:
+                        _ops = commands[sub.base_name].options
+                        _ops.append(sub.to_option())
+                        commands[sub.base_name].options = _ops
+                    else:
+                        commands[sub.base_name] = SlashCommand(None, sub.base_name, options=[sub.to_dict()], guild_ids=sub.guild_ids, default_permission=sub.default_permission, guild_permissions=sub.guild_permissions)
         #endregion
 
-        for command in list(self.commands.values()):
-            if command.guild_ids is not MISSING:        
-                for x in (command.guild_ids or command.guild_permissions.keys()):
-                    if str(x) not in own_guild_ids:
-                        raise InvalidArgument("Client is not in a server with the id '" + str(x) + "'")
-                
-                    if commands["guilds"].get(x) is None:
-                        commands["guilds"][x] = {}
-
-                    if command.guild_permissions is not MISSING:
-                        command.permissions = command.guild_permissions.get(x)
-                    await self.add_guild_command(command, x)
-                    commands["guilds"][x][command.name] = command
-            else:
-                await self.add_global_command(command)
-                commands["globals"][command.name] = command
-
-        for base_name in self.subcommands:
-            subcommands = list(self.subcommands[base_name].values())
-            base = SlashCommand(None, subcommands[0].base_name, options=[x.to_dict() for x in subcommands], guild_ids=subcommands[0].guild_ids, default_permission=subcommands[0].default_permission, guild_permissions=subcommands[0].guild_permissions)
-            if base.guild_ids is not MISSING or base.guild_permissions is not MISSING:
-                for x in (base.guild_ids or base.guild_permissions.keys()):
-                    if str(x) not in own_guild_ids:
-                        raise InvalidArgument("Client is not in a server with the id '" + str(x) + "'")
-
-                    if commands["guilds"].get(x) is None:
-                        commands["guilds"][x] = {}
-                    await self.make_sub_command(base, guild_id=x)
-                    commands["guilds"][x][base.name] = base
-            else:
-                await self.make_sub_command(base)
-                commands["globals"][base.name] = base
-        
-        for base_name in self.subcommand_groups:
-            subcommand_groups = list(self.subcommand_groups[base_name].values())
-            base = SlashCommand(None, subcommand_groups[0].base_names[0], guild_ids=subcommand_groups[0].guild_ids, options=[], default_permission=subcommand_groups[0].default_permission, guild_permissions=subcommand_groups[0].guild_permissions)
+        async def guild_stuff(command, guild_ids):
+            """Adds the command to the guilds"""
+            for x in guild_ids:
+                if str(x) not in own_guild_ids:
+                    raise InvalidArgument("client is not in a server with the id '" + str(x) + "'")
             
-            for x in subcommand_groups:
-                del x.base_names[0]
+                if added_commands["guilds"].get(x) is None:
+                    added_commands["guilds"][x] = {}
+
+                if command.guild_permissions is not MISSING:
+                    command.permissions = command.guild_permissions.get(x)
                 
-                subs: List[dict] = []
-                for _base in x.base_names:
-                    subs.append(SlashOption(OptionType.SUB_COMMAND_GROUP, _base).to_dict())
+                await self.add_guild_command(command, x)
+                added_commands["guilds"][x][command.name] = command
+        async def global_stuff(command):
+            await self.add_global_command(command)
+            added_commands["globals"][command.name] = command
 
-                subs.append(x.to_dict())
-
-                for i in range(0, len(subs) - 1):
-                    subs[i]["options"] = [subs[i + 1]]
-
-                if subs[0]["name"] not in [x["name"] for x in base.options]:
-                    base.options += [subs[0]]
-                else:
-                    b = base.options
-                    b[get_index(b, subs[0]["name"], lambda x: x.get('name'))]["options"].append(subs[1])
-                    base.options = b
-
-            if base.guild_ids is not MISSING or base.guild_permissions is not MISSING:
-                for x in (base.guild_ids or base.guild_permissions.keys()):
-                    if str(x) not in own_guild_ids:
-                        raise InvalidArgument("Client is not in a server with the id '" + str(x) + "'")
-                    
-                    if commands["guilds"].get(x) is None:
-                        commands["guilds"][x] = {}
-
-                    await self.make_sub_command(base, guild_id=x)
-                    commands["guilds"][x][base.name] = base
+        for x in commands:
+            command = commands[x]
+            # guild only command
+            if command.guild_ids is not MISSING:
+                logging.debug("adding '" + str(command.name) + "' as guild_command")
+                await guild_stuff(command, command.guild_ids)
+            # global command with guild permissions
+            elif command.guild_ids is MISSING and command.guild_permissions is not MISSING:
+                logging.debug("adding '" + str(command.name) + "' as guild_command and global")
+                await guild_stuff(command, list(command.guild_permissions.keys()))
+            # global command
             else:
-                await self.make_sub_command(base)
-                commands["globals"][base.name] = base
-
+                logging.debug("adding '" + str(command.name) + "' as global command")
+                await global_stuff(command)
 
         if self.delete_unused:
             api_coms = await self._get_global_commands()
             for apic in api_coms:
-                if commands["globals"].get(apic["name"]) is None:
+                logging.debug("deleting global command '" + str(apic["name"]) + "'")
+                if added_commands["globals"].get(apic["name"]) is None:
                     await delete_global_command(self._discord, apic["id"])
             async for x in self._discord.fetch_guilds():
                 _id = str(x.id)
                 api_coms = await self._get_guild_commands(_id)
                 for apic in api_coms:
-                    if commands["guilds"].get(_id) is None or commands["guilds"][_id].get(apic["name"]) is None:
+                    if None in [added_commands["guilds"].get(_id), added_commands["guilds"][_id].get(apic["name"])]:
+                        logging.debug("deleting guild command '" + str(apic["name"]) + "' in guild " + str(_id))
                         await delete_guild_command(self._discord, apic["id"], _id)
 
-        # print("synchronized slash commands")
+        logging.info("synchronized slash commands")
     
     async def _get_api_command(self, name) -> Union[dict, None]:
         for x in await self._get_commands():
@@ -280,10 +308,11 @@ class Slash():
             try:
                 commands += await get_guild_commands(self._discord, x.id)
             except Forbidden:
-                print("Got forbidden in", x.name, x.id)
+                logging.warn("Got forbidden in " + str(x.name) + " (" + str(x.id) + ")")
                 continue
         return commands
     async def _get_guild_commands(self, guild_id: str) -> List[dict]:
+        logging.debug("getting guild commands in " + str(guild_id))
         return await get_guild_commands(self._discord, guild_id)
     
     async def add_global_command(self, base):
@@ -318,6 +347,8 @@ class Slash():
         """
         target_guild = guild_id
         api_command = await self._get_guild_api_command(base.name, guild_id)
+        if api_command is not None:
+            api_permissions = await get_command_permissions(self._discord, api_command["id"], guild_id)
         # If no command in that guild
         if api_command is None:
             # Check global commands
@@ -326,9 +357,14 @@ class Slash():
             if api_command is not None:
                 await delete_global_command(self._discord, api_command["id"])
             await create_guild_command(base.to_dict(), self._discord, target_guild, base.permissions.to_dict())
+        elif api_command != base or api_permissions != base.permissions:
+            if api_command != base:
+                await edit_guild_command(api_command["id"], self._discord, target_guild, base.to_dict(), base.permissions.to_dict())
+            elif api_permissions != base.permissions:
+                await update_command_permissions(self._discord.user.id, self._discord.http.token, guild_id, api_command["id"], base.permissions.to_dict())
         else:
-        # elif api_command != base:
-            await edit_guild_command(api_command["id"], self._discord, target_guild, base.to_dict(), base.permissions.to_dict())
+            await update_command_permissions(self._discord.user.id, self._discord.http.token, guild_id, api_command["id"], base.permissions.to_dict())
+        # else:
 
     async def make_sub_command(self, base: SlashCommand, guild_id=MISSING):
         """Creates a new sub command and edits it if the base already exsits
@@ -363,14 +399,14 @@ class Slash():
         await delete_guild_commands(self._discord, guild_id)
     async def nuke_commands(self):
         """**Deletes every command for the bot, including globals and commands in every guild**"""
-        print("nuking...")
+        logging.debug("nuking...")
         await self.delete_global_commands()
-        print("nuked global commands")
+        logging.debug("nuked global commands")
         async for guild in self._discord.fetch_guilds():
-            print("nuking commands in", guild.id)
+            logging.debug("nuking commands in" + str(guild.id))
             await self.delete_guild_commands(guild.id)
-            print("nuked commands in", guild.id, guild.name)
-        print("nuked")
+            logging.debug("nuked commands in" + str(guild.name) + " (" + str(guild.id) + ")")
+        logging.info("nuked all commands")
 
 
     def command(self, name, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING):
@@ -392,7 +428,7 @@ class Slash():
                 1-100 character description of the command; default the command name
             options: List[:class:`~SlashOptions`], optional
                 The parameters for the command; default MISSING
-            choices: :class:`[type]`, optional
+            choices: List[:class:`dict`], optional
                 Choices for string and int types for the user to pick from; default MISSING
             guild_ids: List[:class:`str` | :class:`int`], optional
                 A list of guild ids where the command is available; default MISSING
@@ -449,7 +485,7 @@ class Slash():
 
             Note: Replace `default_value` with a value you want to be used if the parameter is not specified in discord, if you don't want a default value, just set it to `None`
             """
-            self.commands[name] = SlashCommand(callback, name, description, options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
+            self.commands[format_name(name)] = SlashCommand(callback, name, description, options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
         return wrapper
     def subcommand(self, base_name, name, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING):
         """A decotator for a subcommand
@@ -524,10 +560,10 @@ class Slash():
 
             Note: Replace `default_value` with a value you want to be used if the parameter is not specified in discord, if you don't want a default value, just set it to `None`
             """
-            if self.subcommands.get(base_name) is None:
-                self.subcommands[base_name] = {}
+            if self.subcommands.get(format_name(base_name)) is None:
+                self.subcommands[format_name(base_name)] = {}
 
-            self.subcommands[base_name][name] = SubSlashCommand(callback, base_name, name, description, options=options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
+            self.subcommands[format_name(base_name)][format_name(name)] = SubSlashCommand(callback, base_name, name, description, options=options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
         return wrapper
     def subcommand_group(self, base_names, name, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING):
         """A decorator for a subcommand group
@@ -548,7 +584,7 @@ class Slash():
                 1-100 character description of the command; default the command name
             options: List[:class:`~SlashOptions`], optional
                 The parameters for the command; default MISSING
-            choices: :class:`[type]`, optional
+            choices: List[:class:`dict`], optional
                 Choices for string and int types for the user to pick from; default MISSING
             guild_ids: List[:class:`str` | :class:`int`], optional
                 A list of guild ids where the command is available; default MISSING
@@ -605,9 +641,14 @@ class Slash():
 
             Note: Replace `default_value` with a value you want to be used if the parameter is not specified in discord, if you don't want a default value, just set it to `None`
             """
-            if self.subcommand_groups.get(base_names[0]) is None:
-                self.subcommand_groups[base_names[0]] = {}
-            self.subcommand_groups[base_names[0]][name] = SubSlashCommandGroup(callback, base_names, name, description, options=options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
+            base = format_name(base_names[0])
+            sub = format_name(base_names[1])
+            if self.subcommands.get(base) is None:
+                self.subcommands[base] = {}
+            if self.subcommands[base].get(sub) is None:
+                self.subcommands[base][sub] = {}
+
+            self.subcommands[format_name(base_names[0])][format_name(base_names[1])][format_name(name)] = SubSlashCommandGroup(callback, base_names, name, description, options=options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
 
         return wrapper
     def user_command(self, name, guild_ids, default_permission=True, guild_permissions = MISSING):
@@ -653,7 +694,7 @@ class Slash():
                 ...
         """
         def wraper(callback):
-            self.context_commands["user"][name] = UserCommand(callback, name, guild_ids, default_permission, guild_permissions)
+            self.context_commands["user"][format_name(name)] = UserCommand(callback, name, guild_ids, default_permission, guild_permissions)
         return wraper
     def message_command(self, name, guild_ids, default_permission=True, guild_permissions = MISSING):
         """Decorator for message context commands in discord.
@@ -698,7 +739,7 @@ class Slash():
                 ...
         """
         def wraper(callback):
-            self.context_commands["message"][name] = MessageCommand(callback, name, guild_ids, default_permission, guild_permissions)
+            self.context_commands["message"][format_name(name)] = MessageCommand(callback, name, guild_ids, default_permission, guild_permissions)
         return wraper
 
 class Components():
@@ -708,8 +749,13 @@ class Components():
     -----------
         client: :class:`discord.Client`
             The main discord client
-        receive_events: :class:`bool`
-            Whether events like components should be received
+
+        auto_defer: Tuple[:class:`bool`, :class:`bool`]
+            Settings for the auto-defer; Default ``(True, False)``
+
+            ``[0]``: Whether interactions should be deferred automatically
+
+            ``[1]``: Whether the deferration should be hidden (True) or public (False)
 
     Example
     ------------------
@@ -751,7 +797,7 @@ class Components():
         async def my_func(component, msg):
             ...
     """
-    def __init__(self, client: com.Bot, receive_events = True):
+    def __init__(self, client: com.Bot, auto_defer=(True, False)):
         """Creates a new compnent listener
         
         Example
@@ -759,32 +805,57 @@ class Components():
         Components(client)
         ```
         """
-        self._listening_components = []
+        self._buffer = bytearray()
+        self._zlib = zlib.decompressobj()
+
+        self.auto_defer: Tuple[bool, bool] = auto_defer if type(auto_defer) in [list, tuple] else (auto_defer, False)
+        self._listening_components: Dict[str, List[function]] = {}
         """A list of components that are listening for interaction"""
-        self._discord = client
-        if receive_events is True:
-            self._discord.add_listener(self._on_socket_response, 'on_socket_response')
+        self._discord: com.Bot = client
+        if discord.__version__.startswith("2"):
+            self._discord.add_listener(self._on_response, "on_socket_raw_receive")
+        elif discord.__version__.startswith("1"):
+            self._discord.add_listener(self._on_response, 'on_socket_response')
     
-    async def _on_socket_response(self, msg):
-        """Will be executed if the bot receives a socket response"""
+    async def _on_response(self, msg):
+        if discord.__version__.startswith("2"):
+            if type(msg) is bytes:
+                self._buffer.extend(msg)
+
+                if len(msg) < 4 or msg[-4:] != b'\x00\x00\xff\xff':
+                    return
+                msg = self._zlib.decompress(self._buffer)
+                msg = msg.decode('utf-8')
+                self._buffer = bytearray()
+                msg = json.loads(msg)
+        
         if msg["t"] != "INTERACTION_CREATE":
             return
         data = msg["d"]
-
+        
         if data["type"] != 3:
             return
-
-        guild = await self._discord.fetch_guild(data["guild_id"])
-        user = discord.Member(data=data["member"], guild=guild, state=self._discord._connection)
         
+        guild = cache_data(data["guild_id"], AdditionalType.GUILD, data, self._discord._connection)
+        user = discord.Member(data=data["member"], guild=guild, state=self._discord._connection)
+        msg = await getResponseMessage(self._discord._connection, data=data, user=user, response=True)
+        
+        interaction = Interaction(self._discord._connection, data, user, msg)
+        if self.auto_defer[0] is True:
+            await interaction.defer(self.auto_defer[1])
+        self._discord.dispatch("interaction_received", interaction)
 
-        msg = await getResponseMessage(self._discord._connection, data=data, application_id=self._discord.user.id, user=user, response=True)
-        component = EphemeralComponent(self._discord.user.id, state=self._discord._connection, user=user, data=data) if data["message"]["flags"] == 64 else msg.interaction_component
+        # Handle auto_defer
+        msg.interaction_component._handle_auto_defer(self.auto_defer)
+        component = msg.interaction_component
 
+        
         # Get listening components with the same custom id
-        _listening_components = [x for x in self._listening_components if data["data"]["custom_id"] == x[1]]
-        if len(_listening_components) == 1:
-            await _listening_components[0][0](component, msg)
+        listening_components = self._listening_components.get(data["data"]["custom_id"])
+        if listening_components is not None:
+            for listening_component in listening_components:
+                await listening_component(component, msg)
+
 
         if data["data"]["component_type"] == 2:
             self._discord.dispatch("button_press", component, msg)
@@ -829,14 +900,10 @@ class Components():
         -------
         :return: Returns the sent message
         :type: :class:`~Message`
-
-        Raises
-        ------
-        :raises: :class:`discord.InvalidArgument`: A passed argument was invalid
         """
 
         if type(channel) not in [discord.TextChannel, int, str]:
-            raise discord.InvalidArgument("Channel must be of type discord.TextChannel")
+            raise WrongType("channel", channel, "discord.TextChannel")
 
         channel_id = channel.id if type(channel) is discord.TextChannel else channel
         payload = jsonifyMessage(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, reference=reference, mention_author=mention_author, components=components)
@@ -847,7 +914,7 @@ class Components():
         if file is MISSING and files is MISSING:
             r = await self._discord.http.request(route, json=payload)
         else:
-            r = await send_files(route, files=files or [file], payload=payload, http=self._discord.http)
+            r = await send_files(route, files=_or(files, [file]), payload=payload, http=self._discord.http)
 
         msg = Message(state=self._discord._get_state(), channel=channel, data=r)
             
@@ -924,12 +991,21 @@ class Components():
 
                         ``component`` and ``message`` are just example names, you can use whatever you want for them
 
-        
+        Example
+        -------
+        @ui.components.listening_component("custom_id")
+        async def callback(component, message):
+            ...
         """
         def wrapper(callback):
-            if len([x for x in self._listening_components if x[1] == custom_id]) > 0:
-                raise Exception("custom_id " + str(custom_id) + " is already in use! Use another custom_id")
-            self._listening_components.append((callback, custom_id))
+            if not inspect.iscoroutinefunction(callback):
+                raise NoAsyncCallback()
+            if len(inspect.signature(callback).parameters) < 2:
+                raise MissingListenedComponentParameters()
+            
+            if self._listening_components.get(custom_id) is None:
+                self._listening_components[custom_id] = []
+            self._listening_components[custom_id].append(callback)
         return wrapper
     
 
@@ -947,16 +1023,23 @@ class UI():
                 ``parse_method``: :class:`int`, optional
                     How the received interaction argument data should be treated; Default ``ParseMethod.AUTO``
 
-
                 ``delete_unused``: :class:`bool`, optional
                     Whether the commands that are not registered by this slash ui should be deleted in the api; Default ``False``
         
                 ``wait_sync``: :class:`float`, optional
                     How many seconds will be waited until the commands are going to be synchronized; Default ``1``
 
+        auto_defer: Tuple[:class:`bool`, :class:`bool`]
+            Settings for the auto-defer; Default ``(True, False)``
+
+            ``[0]``: Whether interactions should be deferred automatically
+
+            ``[1]``: Whether the deferration should be hidden (True) or public (False)
+
+
 
     """
-    def __init__(self, client, slash_options = {"parse_method": ParseMethod.AUTO, "delete_unused": False, "wait_sync": 1}) -> None:
+    def __init__(self, client, slash_options = {"parse_method": ParseMethod.AUTO, "delete_unused": False, "wait_sync": 1, "auto_defer": (True, False)}, auto_defer: Tuple[bool, bool] = (True, False)) -> None:
         """Creates a new ui object
         
         Example
@@ -964,15 +1047,17 @@ class UI():
         UI(client, slash_options={"delete_unused": True, "wait_sync": 2})
         ```
         """
-        self.components = Components(client)
+        self.components = Components(client, auto_defer=auto_defer)
         """For using message components
         
         :type: :class:`~Components`
         """
         if slash_options is None:
-            slash_options = {"resolve_data": True, "delete_unused": False, "wait_sync": 1}
+            slash_options = {"resolve_data": True, "delete_unused": False, "wait_sync": 1, "auto_defer": auto_defer}
+        if slash_options.get("auto_defer") is None:
+            slash_options["auto_defer"] = auto_defer
         self.slash = Slash(client, **slash_options)
         """For using slash commands
         
         :type: :class:`~Slash`
-        """
+        """
\ No newline at end of file
