@@ -1,23 +1,32 @@
-from discord.state import ConnectionState
+from discord_ui.slash.tools import handle_options
+from .errors import InvalidEvent, OutOfValidRange
+from .slash.errors import AlreadyDeferred, EphemeralDeletion
 from .slash.types import ContextCommand, SlashCommand, SubSlashCommand
-from .tools import MISSING
+from .tools import MISSING, setup_logger
 from .http import BetterRoute, jsonifyMessage, send_files
 from .components import ActionRow, Button, LinkButton, SelectMenu, SelectOption
 
 import discord
-from discord.ext import commands as com
-from discord.errors import HTTPException, InvalidArgument
+from discord.errors import HTTPException
+from discord.state import ConnectionState
+
 import typing
 
+logging = setup_logger("discord-ui")
+
+
+class InteractionType:
+    PING = 1
+    APPLICATION_COMMAND = 2
+    MESSAGE_COMPONENT = 3
 
 class Interaction():
-    def __init__(self, application_id, state, data, user=MISSING) -> None:
-        self._application_id = application_id
+    def __init__(self, state, data, user=MISSING, message=None) -> None:
         self._state: ConnectionState = state
 
-        self._deferred = False
-        self._deferred_hidden = False
-        self._responded = False
+        self._deferred: bool = False
+        self._deferred_hidden: bool = False
+        self._responded: bool = False
 
         if user is not MISSING:
             self.member = user
@@ -25,11 +34,24 @@ class Interaction():
             
             :type: :class:`discord.Member`
             """
-        self.interaction = {
-            "token": data["token"],
-            "id": data["id"]
-        }
 
+        self._original_payload = data
+        self.application_id = data["application_id"]
+        self.token = data["token"]
+        """The token for responding to the interaction"""
+        self.id = data["id"]
+        """The id of the interaction"""
+        self.type = data["type"]
+        """The type of the interaction. See :class:`~InteractionType` for more information"""
+        self.version = data["version"]
+        self.data = data["data"]
+        """The passed data of the interaction"""
+        self.channel_id = data["channel_id"]
+        """The channel-id where the interaction was created"""
+        self.guild_id = data["guild_id"]
+        """The guild-id where the interaction was created"""
+        self.message = message
+        """The message of the interaction"""
 
     async def defer(self, hidden=False):
         """This will acknowledge the interaction. This will show the (*Bot* is thinking...) Dialog
@@ -45,14 +67,15 @@ class Interaction():
         
         """
         if self._deferred:
-            raise Exception("interaction was already deferred")
+            raise AlreadyDeferred()
+
         body = {"type": 5}
         
         if hidden is True:
             body["data"] = {"flags": 64}
             self._deferred_hidden = True
         
-        await self._state.http.request(BetterRoute("POST", f'/interactions/{self.interaction["id"]}/{self.interaction["token"]}/callback'), json=body)
+        await self._state.http.request(BetterRoute("POST", f'/interactions/{self.id}/{self.token}/callback'), json=body)
         self._deferred = True
 
     async def respond(self, content=MISSING, *, tts=False, embed=MISSING, embeds=MISSING, file=MISSING, files=MISSING, nonce=MISSING,
@@ -102,16 +125,16 @@ class Interaction():
         
         if ninja_mode:
             try:
-                await self._state.http.request(BetterRoute("POST", f'/interactions/{self.interaction["id"]}/{self.interaction["token"]}/callback'), json={
+                await self._state.http.request(BetterRoute("POST", f'/interactions/{self.id}/{self.token}/callback'), json={
                     "type": 6
                 })
             except HTTPException as x:
                 if "value must be one of (4, 5)" in str(x).lower():
-                    print(x, "The 'ninja_mode' parameter is not supported for slash commands!")
+                    logging.warning(str(x) + "\n" + "The 'ninja_mode' parameter is not supported for slash commands!")
+                    ninja_mode = False
                 else:
                     print(x)
-            finally:
-                return
+                    return
 
         if self._responded:
             await self.send(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, mention_author=mention_author, components=components, hidden=hidden)
@@ -120,10 +143,15 @@ class Interaction():
         
         payload = jsonifyMessage(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, mention_author=mention_author, components=components)
         
-        hide_message = self._deferred_hidden or not self._deferred and hidden 
+        if self._deferred_hidden is hidden:
+            if self._deferred_hidden is False and hidden is True:
+                logging.warning("Your response should be hidden, but the interaction was deferred public. This results in a public response.")
+            if self._deferred_hidden is True and hidden is False:
+                logging.warning("Your response should be public, but the interaction was deferred hidden. This results in a hidden response.")
+        hide_message = self._deferred_hidden or not self._deferred and hidden
 
         if delete_after is not MISSING and hide_message is True:
-            raise InvalidArgument("Cannot delete hidden message (delete_after won't work)")
+            raise EphemeralDeletion()
 
         if hide_message:
             payload["flags"] = 64
@@ -132,25 +160,25 @@ class Interaction():
             await self.defer(hidden=hide_message)
                 
         if not self._deferred:
-            route = BetterRoute("POST", f'/interactions/{self.interaction["id"]}/{self.interaction["token"]}/callback')
+            route = BetterRoute("POST", f'/interactions/{self.id}/{self.token}/callback')
             await self._state.http.request(route, json={
                     "type": 4,
                     "data": payload
                 })
         else:
-            route = BetterRoute("PATCH", f'/webhooks/{self._application_id}/{self.interaction["token"]}/messages/@original')
+            route = BetterRoute("PATCH", f'/webhooks/{self.application_id}/{self.token}/messages/@original')
             r = await self._state.http.request(route, json=payload)
             if hide_message:
                 self._responded = True
-                return EphemeralMessage(state=self._state, channel=self._state.get_channel(int(r["channel_id"])), data=r)
+                return EphemeralMessage(state=self._state, channel=self._state.get_channel(int(r["channel_id"])), data=r, application_id=self.application_id, token=self.interaction["token"])
 
         if file is not MISSING and files is not MISSING:
-            await send_files(route=BetterRoute("PATCH", f"/webhooks/{self._application_id}/{self.interaction['token']}/messages/@original"), 
+            await send_files(route=BetterRoute("PATCH", f"/webhooks/{self.application_id}/{self.token}/messages/@original"), 
                 files=[file] if files is MISSING else files, payload=payload, http=self._state.http)
         
         if not hide_message:
-            responseMSG = await self._state.http.request(BetterRoute("GET", f"/webhooks/{self._application_id}/{self.interaction['token']}/messages/@original"))
-            msg = await getResponseMessage(self._state, data=responseMSG, application_id=self._application_id, response=False)
+            responseMSG = await self._state.http.request(BetterRoute("GET", f"/webhooks/{self.application_id}/{self.token}/messages/@original"))
+            msg = await getResponseMessage(self._state, data=responseMSG, response=False)
             if delete_after is not MISSING:
                 await msg.delete(delete_after)
             return msg
@@ -202,11 +230,11 @@ class Interaction():
         if hidden:
             payload["flags"] = 64
 
-        route = BetterRoute("POST", f'/webhooks/{self._application_id}/{self.interaction["token"]}')
+        route = BetterRoute("POST", f'/webhooks/{self.application_id}/{self.token}')
         r = await self._state.http.request(route, json=payload)
 
         if file is not MISSING and files is not MISSING:
-            await send_files(route=BetterRoute("PATCH", f"/webhooks/{self._application_id}/{self.interaction['token']}/messages/@original"), 
+            await send_files(route=BetterRoute("PATCH", f"/webhooks/{self.application_id}/{self.token}/messages/@original"), 
                 files=[file] if files is MISSING else files, payload=payload, http=self._state.http)
 
         if hidden:
@@ -214,15 +242,17 @@ class Interaction():
 
         return await getResponseMessage(self._state, r, response=False)
 
-        
+    def _handle_auto_defer(self, auto_defer):
+        self._deferred = auto_defer[0]
+        self._deferred_hidden = auto_defer[1]
 
 class SelectedMenu(Interaction, SelectMenu):
     """A :class:`~SelectMenu` object in which an item was selected"""
-    def __init__(self, data, application_id, user, s, state) -> None:
-        Interaction.__init__(self, application_id, state, data)
+    def __init__(self, data, user, s, state, msg) -> None:
+        Interaction.__init__(self, state, data, user, msg)
         SelectMenu.__init__(self, "EMPTY", [SelectOption("EMPTY", "EMPTY")], 0, 0)
         self._json = s.to_dict()
-        self.values: typing.List[SelectOption] = []
+        self.selected_values: typing.List[SelectOption] = []
         """The list of values which were selected
         
         :type: :class:`~SelectOption`
@@ -231,15 +261,15 @@ class SelectedMenu(Interaction, SelectMenu):
         for val in data["data"]["values"]:
             for x in self.options:
                 if x.value == val:
-                    self.values.append(x)
+                    self.selected_values.append(x)
 
         self.member: discord.Member = user
         """The member who selected the value"""
 
 class PressedButton(Interaction, Button):
     """A :class:`~Button` object that was pressed"""
-    def __init__(self, data, application_id, user, b, state) -> None:
-        Interaction.__init__(self, application_id, state, data)
+    def __init__(self, data, user, b, state, message) -> None:
+        Interaction.__init__(self, state, data, user, message)
         Button.__init__(self, "empty", "empty")
         self._json = b.to_dict()
 
@@ -258,8 +288,8 @@ class PressedButton(Interaction, Button):
 class SlashedCommand(Interaction, SlashCommand):
     """A :class:`~SlashCommand` object that was used"""
     def __init__(self, client, command: SlashCommand, data, user, channel, guild_ids = None) -> None:
-        Interaction.__init__(self, client.user.id, client._connection, data, user)
-        SlashCommand.__init__(self, None, "EMPTY", guild_ids)
+        Interaction.__init__(self, client._connection, data, user)
+        SlashCommand.__init__(self, None, "EMPTY", guild_ids=guild_ids)
         self._json = command.to_dict()
         self.member: discord.Member = user
         """The user who created the interaciton"""
@@ -276,7 +306,7 @@ class SlashedSubCommand(SlashedCommand, SubSlashCommand):
 
 class SlashedContext(Interaction, ContextCommand):
     def __init__(self, client, command: ContextCommand, data, user, channel, guild_ids = None) -> None:
-        Interaction.__init__(self, client.user.id, client._connection, data, user)
+        Interaction.__init__(self, client._connection, data, user)
         ContextCommand.__init__(self)
         self._json = command.to_dict()
         self.member: discord.Member = user
@@ -284,7 +314,7 @@ class SlashedContext(Interaction, ContextCommand):
         self.guild_ids = guild_ids
 
 
-async def getResponseMessage(state: ConnectionState, data, application_id = None, user=None, response = True):
+async def getResponseMessage(state: ConnectionState, data, user=None, response = True):
     """
     Async function to get the response message
 
@@ -310,9 +340,12 @@ async def getResponseMessage(state: ConnectionState, data, application_id = None
     """
     channel = state.get_channel(int(data["channel_id"]))
     if response and user:
-        if data["message"]["flags"] == 64:
-            return EphemeralMessage(state=state, channel=channel, data=data["message"])
-        return ResponseMessage(state=state, application_id=application_id, channel=channel, data=data, user=user)
+        if data.get("message") is not None and data["message"]["flags"] == 64:
+            try:
+                return ResponseMessage(state=state, channel=channel, data=data, user=user)
+            except:
+                return EphemeralMessage(state=state, channel=channel, data=data["message"])
+        return ResponseMessage(state=state, channel=channel, data=data, user=user)
 
     return Message(state=state, channel=channel, data=data)
 
@@ -320,7 +353,8 @@ class Message(discord.Message):
     """A fixed :class:`discord.Message` optimized for components"""
     def __init__(self, *, state, channel, data):
         self.__slots__ = discord.Message.__slots__ + ("components", "supressed")
-    
+        self._payload = data
+
         super().__init__(state=state, channel=channel, data=data)
         self.components: typing.List[typing.Union[Button, LinkButton, SelectMenu]] = []
         """The components in the message
@@ -390,13 +424,13 @@ class Message(discord.Message):
             elif component_type == 3:
                 self.components.append(SelectedMenu._fromData(component))
             else:
-                print("unknown component type")
+                logging.warn("ignoring unknown component type " + str(component_type))
 
     def _update(self, data):
         super()._update(data)
         self._update_components(data)
 
-    async def edit(self, *, content=MISSING, embed=MISSING, embeds=MISSING, attachments=MISSING, suppress=MISSING, 
+    async def edit(self, *, content=MISSING, embeds=MISSING, attachments=MISSING, suppress=MISSING, 
         delete_after=MISSING, allowed_mentions=MISSING, components=MISSING):
         """Edits the message and updates its properties
 
@@ -408,8 +442,6 @@ class Message(discord.Message):
         ----------------
         content: :class:`str`
             The new message content
-        embed: :class:`discord.Embed`
-            Embed rich content
         embeds: List[:class:`discord.Embed`]
             The new list of discord embeds
         attachments: List[:class:`discord.Attachment`]
@@ -423,7 +455,7 @@ class Message(discord.Message):
         components: List[:class:`~Button` | :class:`~LinkButton` | :class:`~SelectMenu`]
             A list of components to be included the message
         """
-        payload = jsonifyMessage(content, embed=embed, embeds=embeds, allowed_mentions=allowed_mentions, attachments=attachments, suppress=suppress, flags=self.flags.value, components=components)
+        payload = jsonifyMessage(content, embeds=embeds, allowed_mentions=allowed_mentions, attachments=attachments, suppress=suppress, flags=self.flags.value, components=components)
         if suppress:
             self.suppressed = suppress
         data = await self._state.http.edit_message(self.channel.id, self.id, **payload)
@@ -445,23 +477,23 @@ class Message(discord.Message):
 
         Raises
         ------
-            :raises: :class:`InvalidArgument` : The specified range was out of the possible range of the component rows 
-            :raises: :class:`InvalidArgument` : The specified row was out of the possible range of the component rows 
+            :raises: :class:`discord_ui.errors.OutOfValidRange` : The specified range was out of the possible range of the component rows 
+            :raises: :class:`discord_ui.errors.OutOfValidRange` : The specified row was out of the possible range of the component rows 
         
         """
         comps = []
         if type(row) is range:
             for i, _ in enumerate(self.action_rows):
-                if i > len(self.action_rows) - 1 or i < 0:
-                    raise InvalidArgument("the specified range is invalid! It has to be between 0 and " + str(len(self.action_rows) - 1))
+                if i >= len(self.action_rows) - 1 or i < 0:
+                    raise OutOfValidRange("row[" + str(i) + "]", 0, len(self.action_rows) - 1)
                 for comp in self.action_rows[i]:
                     if i in row:
                         comp.disabled = disable
                     comps.append(comp)
         else:
             for i, _ in enumerate(self.action_rows):
-                if i > len(self.action_rows) - 1 or i < 0:
-                    raise InvalidArgument("the specified row is invalid! It has to be between 0 and " + str(len(self.action_rows) - 1))
+                if i >= len(self.action_rows) - 1 or i < 0:
+                    raise OutOfValidRange("row", 0, len(self.action_rows) - 1)
                 for comp in self.action_rows[i]:
                     if i == row:
                         comp.disabled = disable
@@ -515,7 +547,7 @@ class Message(discord.Message):
     async def wait_for(self, client, event_name, timeout) -> PressedButton: ...
     @typing.overload
     async def wait_for(self, client, event_name, timeout) -> SelectedMenu: ...
-    async def wait_for(self, client, event_name: typing.Literal["select", "button"], custom_id=MISSING, timeout=MISSING) -> typing.Union[PressedButton, SelectedMenu]:
+    async def wait_for(self, client, event_name: typing.Literal["select", "button"], custom_id=MISSING, timeout=None) -> typing.Union[PressedButton, SelectedMenu]:
         """Waits for a message component to be invoked in this message
 
         Parameters
@@ -538,7 +570,7 @@ class Message(discord.Message):
 
         Raises
         ------
-            :raises: :class:`InvalidArgument` : The event name passed was invalid 
+            :raises: :class:`discord_ui.errors.InvalidEvent` : The event name passed was invalid 
 
         Returns
         ----------
@@ -554,25 +586,25 @@ class Message(discord.Message):
 
             return (await client.wait_for('button_press' if event_name.lower() == "button" else "menu_select", check=check, timeout=timeout))[0]
         
-        raise InvalidArgument("Invalid event name, event must be 'button' or 'select', not " + str(event_name))
+        raise InvalidEvent(event_name, ["button", "select"])
 
 
-class ResponseMessage(Interaction, Message):
+class ResponseMessage(Message):
     r"""A message Object which extends the `Message` Object optimized for an interaction component"""
-    def __init__(self, *, state, application_id, channel, data, user):
-        Interaction.__init__(self, application_id, state, data)
+    def __init__(self, *, state, channel, data, user):
         Message.__init__(self, state=state, channel=channel, data=data["message"])
+        self.interaction = Interaction(state, data, user, self)
 
         self.interaction_component = None
 
         if int(data["data"]["component_type"]) == 2:
             for x in self.buttons:
                 if hasattr(x, 'custom_id') and x.custom_id == data["data"]["custom_id"]:
-                    self.interaction_component = PressedButton(data, self._application_id, user, x, self._state)
+                    self.interaction_component = PressedButton(data, user, x, self._state, self)
         elif int(data["data"]["component_type"]) == 3:
             for x in self.select_menus:
                 if x.custom_id == data["data"]["custom_id"]:
-                    self.interaction_component = SelectedMenu(data, self._application_id, user, x, self._state)
+                    self.interaction_component = SelectedMenu(data, user, x, self._state, self)
 
 class WebhookMessage(Message, discord.WebhookMessage):
     def __init__(self, *, state, channel, data):
@@ -604,8 +636,8 @@ class EphemeralComponent(Interaction):
     
     This class only has the custom_id of the used component, the component type and values
     """
-    def __init__(self, application_id, state, user, data) -> None:
-        Interaction.__init__(self, application_id, state, data, user)
+    def __init__(self, state, user, data) -> None:
+        Interaction.__init__(self, state, data, user)
         self.custom_id = data["data"]["custom_id"]
         self.component_type = data["data"]["component_type"]
         if self.component_type == 3:
@@ -613,7 +645,7 @@ class EphemeralComponent(Interaction):
                 def __init__(self, value) -> None:
                     self.name = None
                     self.value = value
-            self.values = [EphemeralValue(x) for x in data["data"]["values"]]
+            self.selected_values = [EphemeralValue(x) for x in data["data"]["values"]]
 
 
 class EphemeralMessage(Message):
@@ -627,28 +659,12 @@ class EphemeralMessage(Message):
     
     If you want to "reply" to id, use the `interaction.send` method instead
     """
-    def __init__(self, *, state, channel, data):
-        if data.get("attachments") is None:
-            data["attachments"] = []
-        if data.get("embeds") is None:
-            data["embeds"] = []
-        if data.get("edited_timestamp") is None:
-            data["edited_timestamp"] = None
-        if data.get("type") is None:
-            data["type"] = 0
-        if data.get("pinned") is None:
-            data["pinned"] = False
-        if data.get("mention_everyone") is None:
-            data["mention_everyone"] = False
-        if data.get("tts") is None:
-            data["tts"] = False
-        if data.get("content") is None:
-            data["content"] = None
+    def __init__(self, *, state, channel, data, application_id=MISSING, token=MISSING):
         Message.__init__(self, state=state, channel=channel, data=data)
-    async def edit(self, **kwargs):
-        raise Exception("Cannot edit ephemeral message")
-    async def delete(self, *, delay):
-        raise Exception("Cannot delete ephemeral message")
-    async def respond(self, **kwargs):
-        raise Exception("Cannot reply to ephemeral message. Instead, use the '.send' function of the interaction")
-    
+        self._application_id = application_id
+        self._interaction_token = token
+    async def edit(self, **fields):
+        r = BetterRoute("PATCH", f"/webhooks/{self._application_id}/{self._interaction_token}/messages/{self.id}")
+        self._update(await self._state.http.request(r, json=jsonifyMessage(**fields)))        
+    async def delete(self):
+        raise EphemeralDeletion()
\ No newline at end of file
